#!/usr/bin/python3

## author: G. Kłopotek

import argparse
import logging
import csv
import os
from alive_progress import alive_bar
from pathlib import Path

import multiprocessing as mp
import datetime
from tqdm import *

"""

"""
def readNGScard(filename: str,csv_name: str, csv_write_mode):

    writer = csv.writer(open(csv_name, csv_write_mode))
    lineCtr=0
    endCtr=0
    line_split_array=[]
    line_read=False
    header_wrote=False
    sequence_no=0
    #logging.info("Dumping %s... \n" % (filename))
    #with codecs.open(filename, 'r', encoding='utf-8',
    #                 errors='ignore') as fp:
    with open(filename) as fp:
        for line in fp:
            skipObs=False
            #line=line.replace("*","")
            #line=line.replace("#INF.....","")
            line_split=line.split()
            #line_split_col_number=len(line_split)
            if(line_split[0]=="$END"):
                endCtr+=1
                continue
            if (endCtr==3):
                line_read=True
            if (line_read):
                sequence_no = line_split[len(line_split)-1][-1:]
            #print(sequence_no)
            if (sequence_no=="1"): # 1st card
                #line_split_array=line_split
                #print(line_split)
                #if (len(line_split)==11):
                #    time_str=line_split[3]+"-"+line_split[4]+"-"+line_split[5]+" "+line_split[5]+":"+line_split[6]+":"+line_split[7]+":"+line_split[8][0]+line_split[8][1]
                #else:
                # parse the line using column width

                #time_str = line_split[3] + "-" + line_split[4] + "-" + line_split[5] + " " + line_split[5] + ":" + \
                #               line_split[6] + ":" + line_split[7] + ":" + line_split[8][0] + line_split[8][1]
                time_str=line[29:33]+"-"+line[34:36].format("%02d")+"-"+line[37:39].format("%02d")+" "+ line[40:42].format("%02d")+ ":" + \
                         line[43:45].format("%02d") +":"+ "{:04.1f}".format(float(line[47:51]))
                #(time_str)
                line_split_array=[time_str,line[0:8],line[10:18],line[20:28]]
                #exit(0)
                header_csvrow = ["datetime", "station1", "station2", "source"]
            if (sequence_no=="2"):
                header_csvrow.append("Observed delay (ns)")
                header_csvrow.append("Observed delay: formal error (ns)")
                header_csvrow.append("Observed delay rate (ps/sec)")
                header_csvrow.append("Observed delay rate formal error (ps/sec)")
                header_csvrow.append("Data quality flag")
                line_split_array += [line[0:20].strip(), line[21:31].strip(), line[31:50].strip(),line[51:61].strip(),line[61:63].strip()]  # line_split[0:6]
                #print([line[0:20].strip(), line[21:31].strip(),line[31:50].strip(),line[51:61].strip(),line[61:63].strip()  ] )
                #exit(0)

            if (sequence_no == "3"):
                header_csvrow.append("Correlation coefficient")
                header_csvrow.append("Formal error for correlation coeficient")
                header_csvrow.append("Fringe amplitude (J)")
                header_csvrow.append("Formal error for fringe amplitude (J)")
                header_csvrow.append("Total fringe phase (radians)")
                header_csvrow.append("Formal error for total fringe phase (radians)")
                line_split_array += line_split[0:6]
                #print(line_split_array)
                #exit(0)
            if (sequence_no == "5"):
                header_csvrow.append("Cable calibration correction (one-way) for site 1 (ns)")
                header_csvrow.append("Cable calibration correction (one-way) for site 2 (ns)")
                line_split_array += line_split[0:2]
            if (sequence_no == "6"):
                header_csvrow.append("Ambient atmospheric temperature at site 1 (deg. C)")
                header_csvrow.append("Ambient atmospheric temperature at site 2 (deg. C)")
                header_csvrow.append("Ambient atmospheric barometric pressure at site 1 (mb)")
                header_csvrow.append("Ambient atmospheric barometric pressure at site 2 (mb)")
                header_csvrow.append("Ambient atmospheric humidity parameter at site 1")
                header_csvrow.append("Ambient atmospheric humidity parameter at site 2")
                line_split_array+=[line[0:10].strip(), line[10:20].strip(),line[20:30].strip(),line[30:40].strip(),line[40:50].strip(),line[50:60].strip()] # line_split[0:6]
                #print(line_split_array)
                #exit(0)
            if (sequence_no == "8"):
                header_csvrow.append("Delay ionosphere correction (ns)")
                header_csvrow.append("Delay ionosphere correction formal error (ns)")
                header_csvrow.append("Delay rate ionosphere correction (ps/s)")
                header_csvrow.append("Delay rate ionosphere correction formal error (ps/s)")
                header_csvrow.append("Ionosphere error flag (0 = ionosphere correction OK)")
                #line_split_array += line_split[0:5]
                line_split_array += [line[0:21].strip(), line[22:30].strip(), line[32:50].strip(), line[52:61].strip(), line[61:64].strip()]  # line_split[0:6]
                #print([line[0:21].strip(), line[22:30].strip(), line[32:50].strip(), line[52:61].strip(),
                #                     line[61:64].strip()] )
                #exit(0)
            #elif (line_read):
            #    line_split_array += line_split[0:7]
            if (sequence_no=="9" and line_read):
                if os.path.getsize(csv_name) == 0 and not header_wrote:
                    #print (os.stat(os.path.abspath(csv_name)).st_size)
                    writer.writerow(header_csvrow)
                    header_wrote=True
                #exit(0)
            #    line_split_array += line_split[0:7]
                #print(line_split_array)
                if not skipObs:
                    writer.writerow(line_split_array)

def readNGSdir(file_absolute: str):

    file=file_absolute.rsplit("/")[-1] # get the name
    #print("reading: ", file)
    if not (file.endswith('.gz')):
        ngscard_version=file.rsplit("_")[1]
        ngscard_band=file.rsplit("_")[0][-2:-1]

        if (ngscard_version== ""):
            ngscard_version = file.rsplit("__")[1]
            ngscard_band = file.rsplit("__")[0][-1]
   #                     #print(ngscard_band)
        output_filename=csv_filename+"_"+ngscard_band+"_"+ngscard_version+"."+csv_filename_ext
                        #print (dirpath + "/" + file,ngscard_version,output_filename)
        #pbar.text("Dumping %s... \n" % (file))
        #readNGScard(dirpath + "/" +file, output_filename, "a")
        readNGScard( file_absolute, output_filename, "a")
        #pbar.text("File content of %s stored in %s \n" % (file, output_filename))


def scanNGSdir(input_dir: str,year_start: int, year_stop: int):
    files, subdirs=[], []

    for f in os.scandir(input_dir):

        if f.is_dir() and (int(f.name) >= year_start and int(f.name)<=year_stop) :
            subdirs.append(f.path)
            #print("Added ",f.name)

        if f.is_file():
            #if os.path.splitext(f.name)[1].lower() in ext:
            files.append(f.path)
            #print("\tAdded", f.name)

    for dir in list(subdirs):
        sf, f = scanNGSdir(dir,year_start,year_stop)
        subdirs.extend(sf)
        files.extend(f)
    return subdirs, files


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Parse the NGS card and store as csv')

    parser.add_argument("-f", "--file", required=False, action="store",type=str, dest="inputfile", help="source file")
    parser.add_argument("-d", "--directory", required=False, action="store", type=str, dest="inputdir",
                        help="source directory")
    parser.add_argument("-p", "--parallel", required=False, action="store", type=int, dest="noOfCores",
                        help="Parallel processing with noOfCores cores")
    parser.add_argument("-t", "--time", required=False, action="store", type=str, dest="time",
                        help="define period, e.g. -t 2001 for starting with year 2001 or -t 2001 2002 to take only two years of data ")
    #parser.add_argument("-o", "--output", required=True, action="store", type=str, dest="output", help="filename of the output", help="filename of the source")
    parser.add_argument("-v", "--verbose", required=False, action="store_true", help="Increase output verbosity")

    args = parser.parse_args()

    if args.verbose:
        logging.basicConfig(level=logging.INFO)

    input_filename=args.inputfile
    input_dir=args.inputdir

    time_period=[]
    year_start=1979
    # currently implemented for years
    if (args.time is not None):
        for i in args.time.rsplit(" "):
            time_period.append(i)
            year_start = int(time_period[0])

    if (len(time_period)>1):
        year_stop=int(time_period[1])
    else:
        year_stop=datetime.datetime.now().year


    csv_filename="output"
    csv_filename_ext="csv"
    #2012-10-01 12:00:00
    #f = open("output", "a")

    output_csv_filename=""

    if (args.noOfCores==None):
        noOfCores = mp.cpu_count()
    else:
        noOfCores = args.noOfCores
    logging.info("Using %s core(s) ... " % noOfCores)

    #exit(0)
    if args.inputfile is not None:
        ngscard_version = input_filename.rsplit("_")[1]
        ngscard_band = input_filename.rsplit("_")[0][-2:-1]
        if (ngscard_version == ""):
            ngscard_version = input_filename.rsplit("__")[1]
            ngscard_band = input_filename.rsplit("__")[0][-1]
        output_filename=input_filename.rsplit("_")[0] + "_"+ngscard_band+"_"+ngscard_version+"."+csv_filename_ext
        readNGScard(input_filename,output_filename,"w")
    elif args.inputdir is not None:
        path = Path(input_dir)
        logging.info("Data period: %d - %d " % (year_start, year_stop))
        logging.info("Counting the number of files in %s... " % input_dir)
        subfolders, fileList = scanNGSdir(input_dir,year_start,year_stop)
        noFiles=len(fileList)
        logging.info("Number of files to process: %d ",noFiles)
        #print(fileList)
        with mp.Pool(processes=noOfCores) as p:
            max_ = noFiles
            with tqdm(total=max_) as pbar:
                for i, _ in enumerate(p.imap_unordered(readNGSdir, fileList)):
                    pbar.update()
    else:
        logging.ERROR("Input soruce file and sourec directory not provided")




