#!/usr/bin/python3

## author: G. Kłopotek

import argparse
import logging
import csv
import os
from pathlib import Path

import multiprocessing as mp
import datetime
from tqdm import *
import netCDF4
import numpy as np
import julian
import fcntl

params2dump=["TempC","AtmPres","RelHum","GroupDelay","GroupRate","DataFlag","QualityCode","GroupDelayFull","Correlation",
             "SBDelay","SNR","Phase","NumGroupAmbig","ClkBrk","DelayTheoretical", "RateTheoretical","AmbigSize"]
stationBasedParams=list()
stationBasedParams.append("TempC")
stationBasedParams.append("AtmPres")
stationBasedParams.append("RelHum")
stationBasedParams.append("ClkBrk")

#eventRelatedParams=list()
#eventRelatedParams.append("ClkBrk")

wrapperType="kall"
wrapperVersion="V004"
IVSAC="iIVS"

"""

"""

import fcntl
import errno

class FileLock:
    def __init__(self, filename=None):
        self.filename = os.path.expanduser('~') + '/LOCK_FILE' if filename is None else filename
        self.lock_file = open(self.filename, 'w+')

    def unlock(self):
        fcntl.flock(self.lock_file, fcntl.LOCK_UN)

    def lock(self, maximum_wait=300):
        waited = 0
        while True:
            try:
                fcntl.flock(self.lock_file, fcntl.LOCK_EX | fcntl.LOCK_NB)
                return True
            except IOError as e:
                if e.errno != errno.EAGAIN:
                    raise e
                else:
                    time.sleep(1)
                    waited += 1
                    if waited >= maximum_wait:
                        return False


class VGOSDBreader:
    def __init__(self,wrappername: str,csv_filename:str):
        print("WRAPPER:",wrappername)
        self.itsWrappername=wrappername.rsplit("/")[-1]
        self.itsWrapperAbsPath=wrappername
        self.itsDirectory= os.path.dirname(self.itsWrapperAbsPath)
        self.itsWrapperversion=self.itsWrappername.rsplit("_")[1]
        self.itsWrapperIVSAC=self.itsWrappername.rsplit("_")[2]
        self.itsWrappername_year=self.itsWrappername[0:2]
        self.itsWrappername_sessionType=self.itsWrappername[7:9]
        self.obs2bsl =""
        self.obs2scan =""
        self.scan2src=""
        self.scan2sta=""
        self.sta2scan= ""
        self.crossrefstatlist=list()
        #self.stationBasedParams=list()
        self.itsSessionName=(self.itsWrappername.rsplit("/")[-1]).rsplit("_")[0]
        self.singleBaseline=False
        self.itsClockBreakStationList=dict()

        if (csv_filename==""):
            self.csv_name=self.itsSessionName+"_"+self.itsWrapperversion+"_"+self.itsWrapperIVSAC+".csv"
        else:
            #self.csv_name = csv_filename + "_"+self.itsWrapperversion+"_"+self.itsWrapperIVSAC+".csv"
            self.csv_name = csv_filename+ "_" + self.itsWrappername_year+ "_"+ self.itsWrappername_sessionType + "_"+self.itsWrapperversion+"_"+self.itsWrapperIVSAC+".csv"
        self.csv_write_mode="a"

        # maps variable with a file name
        self.var2file=dict()
        self.vars2vals=dict()
        self.var2dir=dict()
        # 1) parse the wrapper
        self.parseWrapper()
        # 2) get the stations, sources, observations, etc.
        self.readObsCrossRef()
        self.readSourceCrossRef()
        self.readSourceCrossRef()
        self.itsScan2StationObs=dict()
        self.itsStationObs2Scan=dict()
        self.addedClkBreak=dict()

        self.buildScan2StationObsTable()
        self.buildStationObs2ScanTable()

        self.csv_header_write = False
        self.csvwriter = csv.writer(open(self.csv_name, self.csv_write_mode))
        #self.FileLock=FileLock(self.csv_name)
        self.checkCsvFile()
        self.dumpFiles()
        if any( "ClkBrk" in s for s in params2dump):
            self.getClkBreaks()
        #self.FileLock.lock()
        self.getData()
        #self.FileLock.unlock()


    def checkCsvFile(self):
        if os.path.getsize(self.csv_name) ==0:
            self.csv_header_write=True
        else:
            self.csv_header_write=False

    def dumpFiles(self):
        for _file in self.itsFilelist:
                #print("Dumping ",self.itsDirectory+ "/"+ _file)
                #readVGOSDBfile(_file,csv_filename+"."+csv_filename_ext,"a")
                self.readVGOSDBfile(self.itsDirectory+"/"+_file)

    def readVGOSDBfile(self,filename:str):

        if (os.path.exists(filename)):
            _ncfile = netCDF4.Dataset(filename, 'r')
            _subdir=os.path.relpath(filename,self.itsDirectory).rsplit("/")[0]
            _vars_keys=_ncfile.variables.keys()
            for _var_key in _vars_keys:
                self.var2file[tuple([_subdir,_var_key])]=filename
                self.var2dir[_var_key]=_subdir
                _var_attr = _ncfile.variables[_var_key].ncattrs()
                band_type=""
                var2put=_ncfile[_var_key][:]
                for _ncattr in _var_attr:
                    if (_ncattr =="Band"):
                        band_type=_ncfile.variables[_var_key].getncattr(_ncattr)
                    if (_ncattr=="REPEAT"):
                    #     # then unwrapp the variable
                        #print(filename,_var_key,_ncfile[_var_key][:].size)
                        #print(_ncfile[_var_key][:].fill_value)
                        var2put= self.unfoldVals(_ncfile.variables[_var_key])
                        #print (_var_key," Putting:",var2put)
                        #exit(0)

                    #     repeat_val = _ncfile.variables[_var_key].getncattr(_ncattr)
                    #     nc_var_size = _ncfile[_var_key][:].size
                    #     _nc_file_unwrapped=np.zeros((repeat_val,nc_var_size),).reshape(repeat_val,nc_var_size)
                    #     #print(filename,_var_key,repeat_val,nc_var_size,_nc_file_unwrapped,_ncfile[_var_key][0],print(type(_ncfile[_var_key])))
                    #
                    #     for i,j in zip (range(0,repeat_val),range(0,nc_var_size)):
                    #         #print(i,j,_ncfile[_var_key])
                    #         if(self.getStr(str(_ncfile[_var_key][j]))==''):
                    #             _nc_file_unwrapped[i, j] = ""
                    #         else:
                    #         #    _nc_file_unwrapped[i,j]=_ncfile[_var_key][j]
                    #         #print(_ncfile[_var_key][j])
                    #     #print(_nc_file_unwrapped)
                    #     #exit(0)

                # A key holding directory,variable and band
                self.vars2vals[tuple([_subdir,_var_key,band_type])] = var2put
        #else:
            #logging.warning("Could not find: %s... Skipping " % filename)

    def unfoldVals(self,nc_val):
        _repeat_val = nc_val.getncattr("REPEAT")
        _value=str(nc_val[:]).replace("[","").replace("]","").replace("b","") #.replace(" ","")
        _repeat_val_len=len(nc_val)
        _value_arr = np.zeros((_repeat_val_len),dtype=object)
        #print("*****",_repeat_val,":",_value,"******",_repeat_val_len)
        # split str if more than a single repeat val
        _isStr = False
        if (_repeat_val_len>1):
            for i in range(0,_repeat_val_len):
                #print(_value.split(" ")[1])
                try:
                    _value_arr[i] = float(_value.split(" ")[i])
                    _dtype= float
                    if (_value.split(" ")[i].find(".") == "" ):
                        _dtype=int
                        _value_arr[i] =int(_value.split(" ")[i])
                except ValueError:
                    _value_arr[i] = _value.split(" ")[i].replace("\"","")
                    _isStr=True
        else:
            #could be still string or no
            try:
                _dtype = float
                if (_value.find(".") == ""):
                    _dtype = int
                    _value_arr[0] = int(_value)
                else:
                    _value_arr[0] = float(_value)
            except ValueError:
                _value_arr[0] = _value.replace("\"", "")
                _isStr = True

        #if (_isStr):
        #    tmpStr=""
        #    tmpStr.join(_value_arr)
        #    _value_arr =np.array([tmpStr],dtype=object)
        #print(_value,_repeat_val)
        #exit(0)
        #if (len(_value)>1):
        #    _new_array = np.zeros([_repeat_val,len(_value)],dtype=type(nc_val))
        #    print(_new_array)
        #    for i,j in zip(range(0,_repeat_val),range(0,len(_value))):
        #        print(i,j)
        #        _new_array[i,j]=_value[j]
        #else:

        #try:
        #    float(_value)
        #    _dtype= float
        #    if (_value.find(".") == "" ):
        #        _dtype=int
        #except ValueError:
        #    _value_tmp =""
        #    _value_tmp.join(_value)
        #    _value=_value_tmp
        #    _dtype = str
        #_dtype=float
        #print("Value after conversion: ",_value)
        _new_array = np.zeros((_repeat_val,_repeat_val_len),dtype=object)
        #print(_value_arr[0])
        #print (type(nc_val),_value,len(_value))
        for i in range(0,_repeat_val):
            for j in range(0,_repeat_val_len):
                _new_array[i][j]=_value_arr[j]
                #print(i,j,_new_array[i][j])

        return _new_array


    def getNumberOfObservations(self):
        _ncfile = netCDF4.Dataset(self.itsDirectory + "/Head.nc", "r")
        return _ncfile.variables['NumObs'][0]

    def parseWrapper(self):
        #print("Got: ",wrapperName)
        self.itsStationlist=[]
        _currentDir =""
        self.itsSubdirs =[]
        self.itsFilelist =[]
        with open(self.itsWrapperAbsPath) as wrp:
            for line in wrp:
                line_split=line.split()
                if (line[0] =="!"):
                    continue
                #add subfolder
                if(line_split[0]=="Default_Dir"):
                    #print(line)
                    self.itsSubdirs.append(line_split[1])
                    _currentDir=line_split[1]
                if(line_split[0] == "Begin" and line_split[1] == "Station"):
                    self.itsStationlist.append(line_split[2])
                if (line_split[0].endswith(".nc")):
                    self.itsFilelist.append(_currentDir+"/"+line_split[0])


    def readObsCrossRef(self):
        _ncfile = netCDF4.Dataset(self.itsDirectory+"/CrossReference/ObsCrossRef.nc", "r")
        self.obs2bsl = _ncfile.variables['Obs2Baseline'][:]
        # in the case of a single-baseline sessions
        if self.obs2bsl.size==2:
        #    #self.obs2bsl=self.obs2bsl
            self.singleBaseline=True
        self.obs2scan = _ncfile.variables['Obs2Scan'][:]
        _ncfile.close()

    def readSourceCrossRef(self):
        _ncfile = netCDF4.Dataset(self.itsDirectory+"/CrossReference/SourceCrossRef.nc", "r")
        self.scan2src = _ncfile.variables['Scan2Source'][:]
        _ncfile.close()

    def readSourceCrossRef(self):
        _ncfile = netCDF4.Dataset(self.itsDirectory+"/CrossReference/StationCrossRef.nc", "r")
        self.crossrefstatlist=_ncfile.variables['CrossRefStationList'][:]

        self.scan2sta = _ncfile.variables['Scan2Station'][:]
        self.sta2scan = _ncfile.variables['Station2Scan'][:]
        _ncfile.close()


    def buildScan2StationObsTable(self):
        #print(self.itsWrappername)
        # i - no of scans, j- station
        for  i in range(0, len(self.scan2sta)):
            for j in range(0,len(self.itsStationlist)):
                if (self.scan2sta[i][j]!=0):
                    # store scan name and station as key to get the station-wide obs no
                    self.itsScan2StationObs[tuple([i+1,self.itsStationlist[j]])]=int(self.scan2sta[i][j])-1

    def buildStationObs2ScanTable(self):
        for _statNo in range(0, len(self.itsStationlist)):
            _stationObsNo=0
            for i in range(0, len(self.scan2sta[:][_statNo])):
                    #print(self.sta2scan[i])
                    _currentScanNo=self.sta2scan[i][_statNo]
                    if(_currentScanNo!=0):
                        #print(_currentScanNo,self.itsStationlist[_statNo],_stationObsNo)
                        self.itsStationObs2Scan[tuple([self.itsStationlist[_statNo], _stationObsNo])] = _currentScanNo
                        _stationObsNo+=1
                        #print(self.itsStationlist[_statNo], _statNo,_statObsNo, i)
                    #self.itsStationObs2Scan[tuple([self.itsStationlist[_statNo],_statObsNo])]=i
        #exit(0)

    def getStationObsNo(self,obsNo:int, stationname:str):

        _scanNo=int(self.obs2scan[obsNo])
        _key=tuple([_scanNo,stationname])
        return self.itsScan2StationObs[_key]


    def getVariable(self, variablename: str, subdir: str, obsNo:int, band: str):
            if (subdir==""):
                _target_key=tuple(([self.var2dir[variablename], variablename,band]))
            else:
                _target_key = tuple(([subdir, variablename, band]))

            if  _target_key in self.vars2vals:
                #print(_target_key,obsNo)
                return str(self.vars2vals[_target_key][obsNo])
            else:
                return None

    def getUTCtime(self,obsNo,stationname:str):

        _scanno = int(self.obs2scan[obsNo]) - 1
        _stationObsNo=obsNo

        _index2Use = _scanno
        _subdir="Scan"
        if (stationname!=""):
            _index2Use=_stationObsNo
            _subdir=stationname

        #print(_scanno)
        time_arr=np.array(self.getVariable("YMDHM",_subdir, _index2Use, "")).tolist()
        time_sec=np.array(self.getVariable("Second",_subdir, _index2Use, ""),dtype=float)
        #print(time_arr,time_sec,len(time_arr))

        time_arr_len=len(time_arr)
        _tmp_idx=0# 2-digit year in vgosdb
        _tmp_idx_day=0
        _tmp_idx_hour =0
        _tmp_idx_minute =0
        time_arr_year = int(time_arr[1:3])
        if(time_arr_len==26):
           # to handle 4-digit year format
            _tmp_idx=5
            _tmp_idx_day=_tmp_idx+1
            _tmp_idx_hour = _tmp_idx_day + 3
            _tmp_idx_minute = _tmp_idx_hour+1
            time_arr_year = int(time_arr[1:5])


        if (_tmp_idx==0):
            if (time_arr_year>= 79):
                time_arr_year+=1900
            else:
                time_arr_year+=2000

        #print(time_arr_year,int(time_arr[4+_tmp_idx:6+_tmp_idx]),int(time_arr[7+_tmp_idx_day:9+_tmp_idx_day]),int(time_arr[9+_tmp_idx_hour:12+_tmp_idx_hour]),int(time_arr[13+_tmp_idx_minute:15+_tmp_idx_minute]))
        time_str = str(time_arr_year) + "-" + \
                   "{:02d}".format(int(time_arr[3+_tmp_idx:5+_tmp_idx])) + "-"\
                   + "{:02d}".format(int(time_arr[7+_tmp_idx_day:9+_tmp_idx_day])) + " "\
                   + "{:02d}".format(int(time_arr[9+_tmp_idx_hour:12+_tmp_idx_hour])) + ":" + \
                   "{:02d}".format(int(time_arr[13+_tmp_idx_minute:15+_tmp_idx_minute]))+ ":" +\
                   "{:04.1f}".format(time_sec)
        #print(time_str)
        #exit(0)

        return time_str

    def getStr(self,val_i):

        tmpstr = val_i.replace("b","").replace("[","").replace("]","").replace("'","").replace(" ","")  # .replace("[","") #.replace("]","").replace("b","")
        return tmpstr

    def getStationNames(self,obsNo: int):

        #print(self.obs2scan[obsNo])
        #_scanNo=self.obs2scan[obsNo]-1
        #print(s
        # elf.scan2sta[_scanNo])
        #print(self.obs2bsl[obsNo],self.itsStationlist,self.crossrefstatlist)
        if (self.obs2bsl.size==2):
            _obs2bsl_stat1=0
            _obs2bsl_stat2=1
        else:
            [_obs2bsl_stat1, _obs2bsl_stat2]=self.obs2bsl[obsNo]

        _obs2bsl_stat1-=1
        _obs2bsl_stat2-=1

        _stat1= self.getStr(self.itsStationlist[_obs2bsl_stat1])
        _stat2= self.getStr(self.itsStationlist[_obs2bsl_stat2])

        return _stat1, _stat2

    def getClkBreaksStation(self,stationname:str, stationObsNo: int):

        if stationname in self.itsClockBreakStationList:
            #print(stationname, stationObsNo)
            # first get the time of the observation:
            #_stationObsNo = self.getStationObsNo(obsNo, stationname)
            _obs_utc_station=self.getUTCtime(stationObsNo,stationname)
            _obs_utc_station_prev = self.getUTCtime(stationObsNo-1, stationname)

            fmt = "%Y-%m-%d %H:%M:%S.%f"
            _obs_utc_datetime = datetime.datetime.strptime(_obs_utc_station, fmt)
            #print(_obs_utc_datetime)
            _obs_utc_prev_datetime= datetime.datetime.strptime(_obs_utc_station_prev, fmt)


            for _brk in self.itsClockBreakStationList[stationname]:
                #print(_brk,len(self.itsClockBreakStationList[stationname]))
                #print(_brk,_obs_utc_prev_datetime,_obs_utc_datetime)
                _target_key=tuple([stationname,_brk])
                if  (_brk>= _obs_utc_prev_datetime and _brk < _obs_utc_datetime): # and
                      #  _target_key  not in self.addedClkBreak.keys()):
                    print("\tAdding clk break for %s at %s" %(stationname,_obs_utc_datetime))
                    self.addedClkBreak[tuple([stationname,_brk])]=1
                    return 1

                    #print (stationname,_brk," bigger than",_obs_utc," ",_t_diff.seconds)

            return 0
        else:
            return 0
        #exit(0)

    def getStationData(self,obsNo:int):

        _csv_row=[]
        _csv_header=[]
        _station1,_station2=self.getStationNames(obsNo)

        _obsNoPerStat1 = self.getStationObsNo(obsNo, _station1)
        _obsNoPerStat2 = self.getStationObsNo(obsNo, _station2)

        _obsNoPerStat1_val=0
        _obsNoPerStat2_val=0

        for _param2dump in params2dump:
            #print(_param2dump)
            #if tuple([_station1,_param2dump]) in self.var2file or _param2dump in stationBasedParams:
            if _param2dump in stationBasedParams:
                if _param2dump=="ClkBrk":
                    # returns one if the epoch of the observation is close to the clk break epoch
                    _obsNoPerStat1_val = self.getClkBreaksStation(_station1,_obsNoPerStat1)
                    _obsNoPerStat2_val = self.getClkBreaksStation(_station2,_obsNoPerStat2)
                    #print(_station1, "*",_obsNoPerStat1_val,"*",_station2,"*","*",_obsNoPerStat2_val,"*")
                    if(_obsNoPerStat1_val==1 or _obsNoPerStat2_val==1):
                        print("\t For baseline: %s-%s: %d-%d " % (_station1,_station2,_obsNoPerStat1_val,_obsNoPerStat2_val))
                    _csv_row += [_obsNoPerStat1_val, _obsNoPerStat2_val]
                    _csv_header += [_param2dump + "_Station1", _param2dump + "_Station2"]
                    continue
                #print(_param2dump)
                if tuple([_station1,_param2dump]) in self.var2file:
                    # If there are no station-wide params, put 0
                    _obsNoPerStat1_val= self.getVariable(_param2dump,_station1, _obsNoPerStat1, "")
                    if(_obsNoPerStat1_val==None):
                        _obsNoPerStat1_val=0
                if tuple([_station2,_param2dump]) in  self.var2file:
                    _obsNoPerStat2_val = self.getVariable(_param2dump, _station2, _obsNoPerStat2, "")
                    if(_obsNoPerStat2_val==None):
                        _obsNoPerStat2_val=0
                # in the case of folded data (repeat values), convert str to float
                if(type(_obsNoPerStat1_val)==str):
                    _obsNoPerStat1_val =float((_obsNoPerStat1_val).replace("[","").replace("]",""))
                if(type(_obsNoPerStat2_val)==str):
                    _obsNoPerStat2_val = float((_obsNoPerStat2_val).replace("[","").replace("]",""))

                _csv_row += [_obsNoPerStat1_val, _obsNoPerStat2_val]
                _csv_header+= [_param2dump+"_Station1",_param2dump+"_Station2"]

                #self.stationBasedParams.append(_param2dump)
            #print(_csv_row,_csv_header)
        #exit(0)
        return _csv_header,_csv_row

    def getClkBreaks(self):
        #print("In clk breaks")
        _clkBreakFile="ClockBreak.nc"
        for _file in self.itsFilelist:
            #print(_file)
            if _file.split("/")[1] == _clkBreakFile:
                #print(_file)
                _subdir= _file.split("/")[0]
                _brk_numb=int(self.getStr(str(self.vars2vals[tuple([_subdir,"BRK_NUMB",""])])))
                _clkbreak=self.getStr(str(self.vars2vals[tuple([_subdir,"CLKBREAK",""])])).split(",")
                _clkbreakStatList=self.getStr(str(self.vars2vals[tuple([_subdir,"ClockBreakStationList",""])])).replace("\n",",").replace("\"","").split(",")
                #_clkbreakStatList=self.vars2vals[tuple([_subdir,"ClockBreakStationList",""])]
                _clkbreakEpoch=str(self.vars2vals[tuple([_subdir,"ClockBreakEpoch",""])]).replace("[","").replace("]","").strip().split() #.replace("  ","") #.split(",")
                #print(_brk_numb,_clkbreak,_clkbreakStatList,_clkbreakEpoch)
                for i in range(0,_brk_numb):
                    _stat=_clkbreakStatList[i]
                    _stat_clkEpoch=float(_clkbreakEpoch[i]) # mjd
                    #_stat_clkEpoch_jd=_stat_clkEpoch # -0.5 #+ 2400000.5 # to jd

                    #_stat_clkEpoch_year,_stat_clkEpoch_month,_stat_clkEpoch_day  =jd_to_date(_stat_clkEpoch_jd)

                    #_stat_clkEpoch_sod=_stat_clkEpoch_day-int(_stat_clkEpoch_day)
                    #_stat_clkEpoch_day=math.floor(_stat_clkEpoch_day)
                    #_stat_clkEpoch_hour=math.floor(_stat_clkEpoch_sod*86400/3600)
                    #_stat_clkEpoch_minute=math.floor((_stat_clkEpoch_sod*86400/3600-_stat_clkEpoch_hour)*60)
                    #_stat_clkEpoch_seconds=round(((_stat_clkEpoch_sod*86400/3600-_stat_clkEpoch_hour)*60)-_stat_clkEpoch_minute)*60
                    #if(_stat_clkEpoch_seconds==60):
                    #    _stat_clkEpoch_seconds=0
                    #    _stat_clkEpoch_minute+=1
                    #if (_stat_clkEpoch_minute == 60):
                    #    _stat_clkEpoch_seconds=0
                    #    _stat_clkEpoch_minute=0
                    #    _stat_clkEpoch_hour+=1

                    #print(_stat_clkEpoch_year,_stat_clkEpoch_month,_stat_clkEpoch_day,_stat_clkEpoch_hour,_stat_clkEpoch_minute,_stat_clkEpoch_seconds)
                    #_stat_clkEpoch_str=str(_stat_clkEpoch_year)+"-"+"{:02d}".format(_stat_clkEpoch_month)+"-"+"{:02d}".format(_stat_clkEpoch_day)+" "+\
                    #                   "{:02d}".format(_stat_clkEpoch_hour)+":"+"{:02d}".format(_stat_clkEpoch_minute)+":"+"{:02d}".format(_stat_clkEpoch_seconds)
                    #_stat_clkEpoch_sod=
                    #print(_stat_clkEpoch,_stat_clkEpoch_str)
                    _stat_clkEpoch_mjd=_stat_clkEpoch
                    _datestd=0
                    if (int(_stat_clkEpoch_mjd)<2400000.5):
                        _datestd=julian.from_jd(_stat_clkEpoch_mjd, fmt='mjd')
                    else:
                        _datestd=julian.from_jd(_stat_clkEpoch_mjd-2400000.5, fmt='mjd')
                    #fmt ="%Y-%m-%d %H:%M:%S"
                    #_datestd = datetime.datetime.strptime(_stat_clkEpoch_str, fmt) #.date()
                    #print(_datestd)
                    print ("Found clk break: for: %s at %s" %(_stat,_datestd))
                    if _stat in self.itsClockBreakStationList:
                        self.itsClockBreakStationList[_stat].append(_datestd)
                    else:
                        self.itsClockBreakStationList[_stat]=[]
                        self.itsClockBreakStationList[_stat].append(_datestd)

        #exit(0)
    def getData(self):

        # retrieves all the necessary data heer
        for i in range(0,self.getNumberOfObservations()):
            # first get the time
            #print("Processing ",i,"/",self.getNumberOfObservations())
            #print(self.getUTCtime(i,""))
            _csv_row=[self.getUTCtime(i,"")]
            _csv_header=["UTC Time"]
            # add session name
            _csv_row += [self.itsSessionName]
            _csv_header += ["SessionName"]
            # add scan no
            _scanNo = int(self.obs2scan[i])
            _csv_row += [_scanNo]
            _csv_header += ["ScanNo"]
            # add station names
            _station1,_station2=self.getStationNames(i)
            _csv_row+=[_station1,_station2]
            _csv_header+=["Station1", "Station2"]
            # add Source
            src_val_i=self.getVariable("Source","", i,"")
            val_i = self.getStr(src_val_i)
            _csv_row += [val_i]
            _csv_header+=["Source"]
            # add station-specific data
            _csv_header_stat,_csv_row_stat=self.getStationData(i)
            _csv_row+=_csv_row_stat
            _csv_header+=_csv_header_stat

            # Finally, add X/S band params
            for param2dump in params2dump:
                _subdir=""
                for band in ["X","S",""]:
                    #print(band,param2dump)
                    if not any( param2dump in s for s in stationBasedParams):
                    #if not any(param2dump in s for s in self.itsStationlist) and not any(param2dump in s for s in self.stationBasedParams):

                        val_i=self.getVariable(param2dump,_subdir,i,band)

                        if val_i is not None:
                            _csv_row+=[self.getStr(self.getVariable(param2dump,_subdir,i,band))]
                            #print(self.getVariable(param2dump,_subdir,i,band))
                            if (band != ""):
                                _csv_header+=[param2dump+"_"+band]
                            else:
                                _csv_header += [param2dump]


            if self.csv_header_write:
            #print (os.stat(os.path.abspath(self.csv_name)).st_size)
                self.csvwriter.writerow(_csv_header)
                self.csv_header_write = False
            self.csvwriter.writerow(_csv_row)


def scanVGOSDBdir(input_dir: str,year_start: int, year_stop: int):
    files, subdirs=[], []

    for f in os.scandir(input_dir):

        if f.is_dir() and str(f.name[0:2]).isnumeric():
            _f_dir_name=f.name
            _f_dir_name_year=int(_f_dir_name[0:2])
            if (_f_dir_name_year<79):
                _f_dir_name_year+=2000
            else:
                _f_dir_name_year+=1900
            if (_f_dir_name_year >= year_start and _f_dir_name_year <=year_stop) :
                subdirs.append(f.path)
                #print("Added ",f.name)

        if f.is_file():
            if f.name.split(".")[1]=="wrp":
                _f_name_split=f.name.split(".")[0].split("_")
                if(_f_name_split[1]==wrapperVersion and _f_name_split[2]==IVSAC and _f_name_split[3]==wrapperType):
                    files.append(f.path)
                    #print("\tAdded file: ", f.name)


    for dir in list(subdirs):
        sf, f = scanVGOSDBdir(dir,year_start,year_stop)
        subdirs.extend(sf)
        files.extend(f)
        #print(f)
    return subdirs, files

def readVGOSDBdirs(inputWrapper: str):

    vgosdbreader = VGOSDBreader(inputWrapper,csv_filename)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Parse the NGS card and store as csv')

    parser.add_argument("-f", "--file", required=False, action="store",type=str, dest="inputwrapper", help="source database")
    parser.add_argument("-d", "--directory", required=False, action="store", type=str, dest="inputdir",
                        help="source directory")
    parser.add_argument("-p", "--parallel", required=False, action="store", type=int, dest="noOfCores",
                        help="Parallel processing with noOfCores cores")
    parser.add_argument("-t", "--time", required=False, action="store", type=str, dest="time",
                        help="define period, e.g. -t 2001 for starting with year 2001 or -t 2001 2002 to take only two years of data ")
    #parser.add_argument("-o", "--output", required=True, action="store", type=str, dest="output", help="filename of the output", help="filename of the source")
    parser.add_argument("-v", "--verbose", required=False, action="store_true", help="Increase output verbosity")



    args = parser.parse_args()

    if args.verbose:
        logging.basicConfig(level=logging.INFO)

    input_wrapper=args.inputwrapper
    input_dir=args.inputdir

    time_period=[]
    year_start=1979
    # currently implemented for years
    if (args.time is not None):
        for i in args.time.rsplit(" "):
            time_period.append(i)
            year_start = int(time_period[0])

    if (len(time_period)>1):
        year_stop=int(time_period[1])
    else:
        year_stop=datetime.datetime.now().year


    csv_filename="output"
    csv_filename_ext="csv"
    #2012-10-01 12:00:00
    #f = open("output", "a")

    output_csv_filename=""

    if (args.noOfCores==None):
        noOfCores = mp.cpu_count()
    else:
        noOfCores = args.noOfCores
    logging.info("Using %s core(s) ... " % noOfCores)

    #exit(0)
    if args.inputwrapper is not None:

        vgosdbreader= VGOSDBreader(input_wrapper,"")

    elif args.inputdir is not None:
        path = Path(input_dir)
        logging.info("Data period: %d - %d " % (year_start, year_stop))
        logging.info("Counting the number of files in %s... " % input_dir)

        subfolders, fileList =scanVGOSDBdir(input_dir,year_start,year_stop)
        noFiles=len(fileList)
        logging.info("Number of files to process: %d ",noFiles)
        #exit(0)
        print(fileList)
        #exit(0)
        with mp.Pool(processes=noOfCores) as p:
            max_ = noFiles
            with tqdm(total=max_) as pbar:
                for i, _ in enumerate(p.imap_unordered(readVGOSDBdirs, fileList)):
                    pbar.update()
    else:
        logging.ERROR("Input soruce file and sourec directory not provided")




